/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#ifndef __RTCachedString_h__
#define __RTCachedString_h__ included

#ifdef PRAGMA
#pragma interface
#endif

#ifndef __RTConfig_h__
#include <RTConfig.h>
#endif

class RTCachedString
{
public:
    RTS_INLINE              RTCachedString( void );
    explicit                RTCachedString( const char * );
			    RTCachedString( const RTCachedString & );
			    ~RTCachedString( void );

    RTS_INLINE operator     const char * ( void ) const;

    RTCachedString        & operator=( const char * );
    RTCachedString        & operator=( const RTCachedString & );

    RTS_INLINE int          operator==( const char * ) const;
    RTS_INLINE int          operator!=( const char * ) const;
    RTS_INLINE int          operator==( const RTCachedString & ) const;
    RTS_INLINE int          operator!=( const RTCachedString & ) const;

    struct Entry
    {
	char      * value;
	unsigned    use_count;
    };

    struct Block
    {
	enum { Size = 32 };

	Entry       entry[ Size ];
	Block     * next;
    };

private:
    Entry                 * _entry;

    static unsigned         _size;
    static unsigned         _used;
    static Entry         ** _contents;
    static Block          * _blocks;

    static Entry          * add( const char * );
    static void             link( Entry * );
    static void             unlink( Entry * );

    static void             lock( void );
    static void             unlock( void );
};

#if RTS_INLINES
#include <RTCachedString.inl>
#endif

#endif // __RTCachedString_h__
