/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#ifndef __RTComponentDescriptor_h__
#define __RTComponentDescriptor_h__ included

#ifdef PRAGMA
#pragma interface
#endif

#ifndef __RTFieldOffset_h__
#include <RTFieldOffset.h>
#endif

struct RTActorClass;
struct RTBindingDescriptor;
struct RTInterfaceDescriptor;

// This structure describes one component of an actor class.
//
// Field            Meaning
// -----            -------
// name             The design name of the component.
// defaultClass     The stereotype of the component.
// offset           The byte offset of the associated RTActorRef within the
//                  containing class, relative to RTActor.
// id               An integer, uniquely identifying this component within
//                  the containing class.
// kind             Distinguishes between fixed, optional and imported
//                  components.
// substitutable    Should classes other than the default be considered
//                  for this component?
// replication      The replication factor of the component.
// numInterfaces    The number of interface requirements.
// interfaces       An array of interface (replication) requirements.
// numBindings      The number of bindings to this component.
// bindings         An array of interface (protocol) requirements.

struct RTComponentDescriptor
{
    enum Kind { Fixed, Optional, Imported };

    const char                    * name;
    const RTActorClass            * defaultClass;
    RTFieldOffset                   offset;
    short                           id;
    char /* Kind */                 kind;
    char /* boolean */              substitutable;
    int                             replication;

    int                             numInterfaces;
    const RTInterfaceDescriptor   * interfaces;

    int                             numBindings;
    const RTBindingDescriptor     * bindings;
};

#endif // __RTComponentDescriptor_h__
