/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#ifndef __RTConnector_h__
#define __RTConnector_h__ included

#ifdef PRAGMA
#pragma interface
#endif

#ifndef __RTActor_h__
#include <RTActor.h>
#endif

#ifndef __RTCachedString_h__
#include <RTCachedString.h>
#endif

#ifndef __RTLayerData_h__
#include <RTLayerData.h>
#endif

extern const RTActorClass RTConnector;

class RTConnector_Actor : public RTActor
{
public:
		 RTConnector_Actor( RTController *, RTActorRef * );
    virtual      ~RTConnector_Actor( void );

    virtual void enumerate( void ) const = 0;

    // The implementations of these two operations use the following six
    // operations which are similar to the old API. Subclasses are encouraged
    // to provide their own implementations of these operations as this will
    // likely yield a more efficient solution.

    virtual RTLayerData * _registerSAP( RTController * _context,
					RTProtocol   * _sap,
					const char   * _name );

    virtual RTLayerData * _registerSPP( RTController * _context,
					RTProtocol   * _spp,
					const char   * _name );

    // The following six operations provide (near) backward compatibility.
    // RTProtocol::getRegisteredName will return nil during the registration
    // operations which is why they have an additional parameter.

    virtual int  registerSAP  ( RTController * _context,
				RTProtocol   * _sap,
				const char   * _name ) = 0;

    virtual void resizeSAP    ( RTController * _context,
				RTProtocol   * _sap,
				int            _old_size ) = 0;

    virtual void deregisterSAP( RTController * _context,
				RTProtocol   * _sap ) = 0;

    virtual int  registerSPP  ( RTController * _context,
				RTProtocol   * _spp,
				const char   * _name ) = 0;

    virtual void resizeSPP    ( RTController * _context,
				RTProtocol   * _spp,
				int            _old_size ) = 0;

    virtual void deregisterSPP( RTController * _context,
				RTProtocol   * _spp ) = 0;

    static const RTActor_class rtg_class;

private:
    class SapData : public RTLayerData
    {
    public:
			     SapData( RTConnector_Actor *,
				      RTProtocol        *,
				      const char        * );
	virtual              ~SapData( void );

	virtual const char * getRegisteredName( void ) const;

	virtual void         resize( RTController * context, int old_size );

	virtual void         deregister( RTController * context );

    private:
	RTConnector_Actor  * _connector;
	RTProtocol         * _sap;
	RTCachedString       _name;
    }; //lint !e1712

    class SppData : public RTLayerData
    {
    public:
			     SppData( RTConnector_Actor *,
				      RTProtocol        *,
				      const char        * );
	virtual              ~SppData( void );

	virtual const char * getRegisteredName( void ) const;

	virtual void         resize( RTController * context, int old_size );

	virtual void         deregister( RTController * context );

    private:
	RTConnector_Actor  * _connector;
	RTProtocol         * _spp;
	RTCachedString       _name;
    }; //lint !e1712

    // unavailable methods
    RTConnector_Actor( const RTConnector_Actor & );
    RTConnector_Actor & operator=( const RTConnector_Actor & );
}; //lint !e1712

#endif // __RTConnector_h__
