/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#ifndef __RTDataObject_h__
#define __RTDataObject_h__ included

#ifdef PRAGMA
#pragma interface
#endif

#ifndef __RTVersionId_h__
#include <RTVersionId.h>
#endif

class  RTDataObject;
class  RTDecoding;
class  RTEncoding;
struct RTObject_class;

typedef RTDataObject * (* RTDataObjectFunc)( void );

class RTDataObject
{
public:
				    RTDataObject( void );
				    RTDataObject( const RTDataObject & );
    virtual                         ~RTDataObject( void );

    virtual int                     operator==( const RTDataObject & ) const;
    RTS_INLINE int                  operator!=( const RTDataObject & ) const;
    virtual RTDataObject          * copy( void ) const;

#if OBJECT_ENCODE
    char                          * printString( void ) const;
    int                             printString( char * buf, int size ) const;
    virtual int                     put( RTEncoding & ) const;
#endif

#if OBJECT_DECODE
    virtual int                     ObjectForm( RTDecoding & );
#endif

    virtual const RTObject_class  * getClassData( void ) const;
    virtual const char            * getTypeName( void ) const;
    RTVersionId                     getVersion( void ) const;
    int                             isType( const char * ) const;
    int                             isSameType( const RTObject_class * ) const;

    static  const RTObject_class    classData;

#define RTType_RTDataObject  (  RTDataObject::classData )

#if OBJECT_DECODE
    static void                     initialize( void );
#endif
};

extern const RTDataObject EmptyObject;

struct RTTypedValue_RTDataObject
{
    const void           * data;
    const RTObject_class * type;

    RTS_INLINE RTTypedValue_RTDataObject( const RTDataObject & ); //lint !e1931
    RTS_INLINE RTTypedValue_RTDataObject( const RTDataObject   &,
					  const RTObject_class * );
}; //lint !e1712

typedef RTDataObject * RTpRTDataObject;

extern const RTObject_class RTType_RTpRTDataObject;

// Helper function for descriptors of classes in the RTDataObject hierarchy.
extern void RTDataObject_destroy( const RTObject_class *, void * );

#if RTS_INLINES
#include <RTDataObject.inl>
#endif

#endif // __RTDataObject_h__
