/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#ifndef __RTTiming_h__
#define __RTTiming_h__ included

#ifdef PRAGMA
#pragma interface
#endif

#ifndef __RTGlobalSignal_h__
#include <RTGlobalSignal.h>
#endif

#ifndef __RTPriority_h__
#include <RTPriority.h>
#endif

#ifndef __RTProtocol_h__
#include <RTProtocol.h>
#endif

#ifndef __RTTimespec_h__
#include <RTTimespec.h>
#endif

class  RTTimerId;
class  RTTimerNode;
struct RTTypedValue;

struct Timing
{
    class Base : public RTProtocol
    {
    public:
	RTS_INLINE Base( void );
	RTS_INLINE ~Base( void );

	static const RTProtocolDescriptor rt_class;

	enum
	{
	    rti_timeout = rtiLast_RTProtocol + 1
	};

	RTS_INLINE RTInSignal timeout( void );

	RTTimespec       currentTime( void ) const;

	RTTimerNode    * informAt( const RTTimespec     & when,
				   const void           * data,
				   const RTObject_class * type,
				   int                    prio = General );

	RTS_INLINE
	RTTimerNode    * informAt( const RTTimespec   & when,
				   int                  prio = General );

	RTS_INLINE
	RTTimerNode    * informAt( const RTTimespec   & when,
				   const RTDataObject & data,
				   int                  prio = General );

	RTS_INLINE
	RTTimerNode    * informAt( const RTTimespec   & when,
				   const RTTypedValue & info,
				   int                  prio = General );

	RTS_INLINE
	RTTimerNode    * informIn( const RTTimespec     & delta,
				   const void           * data,
				   const RTObject_class * type,
				   int                    prio = General );

	RTS_INLINE
	RTTimerNode    * informIn( const RTTimespec   & delta,
				   int                  prio = General );

	RTS_INLINE
	RTTimerNode    * informIn( const RTTimespec   & delta,
				   const RTDataObject & data,
				   int                  prio = General );

	RTS_INLINE
	RTTimerNode    * informIn( const RTTimespec   & delta,
				   const RTTypedValue & info,
				   int                  prio = General );

#if RTS_COMPATIBLE <= 640
	RTTimerNode    * informIn( unsigned long        centiseconds,
				   int                  prio = General );

	RTTimerNode    * informIn( unsigned long        centiseconds,
				   const RTDataObject & data,
				   int                  prio = General );

	RTTimerNode    * informIn( unsigned long        centiseconds,
				   const RTTypedValue & info,
				   int                  prio = General );
#endif // RTS_COMPATIBLE <= 640

	RTS_INLINE
	RTTimerNode    * informEvery( const RTTimespec     & delta,
				      const void           * data,
				      const RTObject_class * type,
				      int                    prio = General );

	RTS_INLINE
	RTTimerNode    * informEvery( const RTTimespec   & delta,
				      const RTDataObject & data,
				      int                  prio = General );

	RTS_INLINE
	RTTimerNode    * informEvery( const RTTimespec   & delta,
				      const RTTypedValue & info,
				      int                  prio = General );

	RTS_INLINE
	RTTimerNode    * informEvery( const RTTimespec & delta );

	int              timeouts( const RTTimerId & );
	int              cancelTimer( RTTimerId & );

	// If ever the real-time clock needs to be adjusted these functions
	// must be used to bracket the change so that the run-time system can
	// be aware of that event and make any necessary adjustments to its
	// internal data structures so that relative timeouts (requested via
	// one of the informIn methods or one of the informEvery methods) are
	// not delayed nor advanced.

	void             adjustTimeBegin( void );
	void             adjustTimeEnd( const RTTimespec & delta );

    protected:
	RTTimerNode    * informIn( const RTTimespec     & when,
				   const RTTimespec     & repeatInterval,
				   const void           * data,
				   const RTObject_class * type,
				   int                    prio );

	enum
	{
	    rtiLast_Timing = rti_timeout
	};

    private:
	friend class RTTimerActor;
	friend class RTTimerNode;

	RTTimerNode * outstanding;

	// unavailable methods
	Base( const Base & );
	Base & operator=( const Base & );
    };
};

#if RTS_INLINES
#include <RTTiming.inl>
#endif

#endif // __RTTiming_h__
