/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTActorRef.h>
#include <RTActorSlot.h>
#include <RTComponentDescriptor.h>
#include <RTImportList.h>

RTActor ** RTActorRef::access( int index ) const
{
    RTActorRef * const this_       = RTConstCast( RTActorRef, this );
    int                replication = _info->replication;

    if( index < 0 || index >= replication )
	return (RTActor **)0;

    switch( _info->kind )
    {
    case RTComponentDescriptor::Imported:
    {
	RTImportList * slot;

	if( replication == 1 )
	    slot = (RTImportList *)_contents;
	else
	{
	    RTImportList::Unused * head = (RTImportList::Unused *)_contents;

	    slot  = (RTImportList *)(void *)&head[ 1 ];
	    slot += index;
	}

	if( slot->used.reference != this )
	    return (RTActor **)0;
	else
	    return &slot->used.actor;
    }

    case RTComponentDescriptor::Optional:
	if( replication == 1 )
	    return (RTActor **)&this_->_contents;
	else if( RTSmallOptional::supports( replication ) != 0 )
	{
	    RTSmallOptional * slot = (RTSmallOptional *)_contents;

	    slot += index + 1;

	    if( slot->inuse() == 0 )
		return (RTActor **)0;
	    else
		return &slot->_actor;
	}
	else
	{
	    RTLargeOptional * slot = (RTLargeOptional *)_contents;

	    slot += index + 1;

	    if( slot->next != (RTLargeOptional *)0 )
		return (RTActor **)0;
	    else
		return &slot->actor;
	}

    case RTComponentDescriptor::Fixed:
	if( replication == 1 )
	    return (RTActor **)&this_->_contents;
	else
	    return &((RTActor **)_contents)[ index ];

    default:
	return (RTActor **)0;
    }
}
