/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTActor.h>
#include <RTActorProbe.h>
#include <RTController.h>
#include <RTDebugger.h>
#include <RTMessage.h>
#include <RTProtocol.h>

int RTProtocol::End::invoke( RTMessage            * replyBuffer,
			     RTController         * invoker,
			     RTProtocol           * port,
			     int                    index,
			     int                    signal,
			     const void           * data,
			     const RTObject_class * type )
{
    RTActor             * remoteActor;
    RTController::Error   error;

    // check if port is bound
    switch( state )
    {
    default:
    case Unbound:
	error = RTController::noConnect;
	remoteActor = (RTActor *)0;
	break;

    case BoundXThread:
	error = RTController::xRtsInvoke;
	remoteActor = (RTActor *)0;
	break;

    case Bound:
	remoteActor = remote->getOwner();

	if( remoteActor->isActive() != 0 )
	    error = RTController::recursiveInvoke;

#if SUSPEND
	else if( remoteActor->isSuspended() != 0 )
	    // suspended actors effectively just defer all messages
	    error = RTController::deferInvoke;
#endif

	else
	    error = RTController::ok;
	break;
    }

    // Mark the reply as invalid. If there is a reply this will change.
    replyBuffer->setInvalid();

    RTMessage * oldReplyBuffer = invoker->setReplyBuffer( replyBuffer );
    RTMessage   msg;

    msg.setData( data, type );
    msg.setDestination( remoteActor, remote, remoteIndex );
    msg.setPriority( System );

    if( signalMap != (const int *)0 )
	msg.setSignal( signalMap[ signal - 1 ] );
    else
	msg.setSignal( signal );

#if OBSERVABLE
    invoker->debugger()->setTimestamp( &msg, port, index );

    RTActor * owner = port->getOwner();

    // the invoked message
    if( owner->probe != (RTActorProbe *)0 )
	owner->probe->evtMessageOut( &msg, RTEventInfo::SendInvoke );
#endif

    if( error != RTController::ok )
    {
	invoker->setError( error,
			   RTController::CommInvoke, port, &index );
    }
    else if( msg.deliver() == RTMessage::Rejected )
    {
	// error already set
    }
    else if( replyBuffer->isValid() == 0 )
    {
	invoker->setError( RTController::noReply,
			   RTController::CommInvoke, port, &index );
    }
    else
    {
#if OBSERVABLE
	// the reply to the invoke
	if( owner->probe != (RTActorProbe *)0 )
	{
	    replyBuffer->setDestination( owner, port, index );
	    owner->probe->evtMessageIn( replyBuffer );
	}
#endif
	invoker->restoreReplyBuffer( oldReplyBuffer );

	return 1;
    }

    invoker->restoreReplyBuffer( oldReplyBuffer );

    return 0;
}
