/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTFieldOffset.h>
#include <RTMemoryUtil.h>
#include <RTinet.h>
#include <RTtcp.h>

extern "C" int RTinet_primary( int sock, RTinet_address * address )
{
    if( sock == INET_NO_SOCKET )
	return 0;

    struct ifconf ifc;
    struct ifreq  ifr;
    struct ifreq  buf[ 20 ];

    ifc.ifc_len = sizeof( buf );
    ifc.ifc_req = buf;

    // get list of configurations of all interfaces in the host
    if( INET_ioctl( sock,
		    SIOCGIFCONF, //lint !e569 !e912 !e917
		    (int *)&ifc ) != 0 ) //lint !e740
	return 0;

    int          avail;
    const char * cursor;
    int          len    = (int)sizeof( ifr );

    for( cursor = ifc.ifc_buf, avail  = ifc.ifc_len;
			       avail >= (int)sizeof( ifr );
	 cursor += len,        avail -= len )
    {
	RTMemoryUtil::memcpy( &ifr, cursor, (int)sizeof( ifr ) );

#if INET_HaveLength
	len = ifr.ifr_addr.sa_len + (int)RTOffsetOf( struct ifreq, ifr_addr );
#endif

#if defined( IFF_LOOPBACK ) || defined( IFF_UP ) || defined( IFF_RUNNING )
	// Only issue the ioctl where these flags exist.
	// skip an interface if we can't get its flags
	if( INET_ioctl( sock,
			SIOCGIFFLAGS, //lint !e569 !e912 !e917
			(int *)&ifr ) != 0 ) //lint !e740
	    continue;

	// skip loopback interfaces
	if( ( ifr.ifr_flags & IFF_LOOPBACK ) != 0 )
	    continue;

	// skip interfaces that are not UP and RUNNING
	if( ( ifr.ifr_flags & ( IFF_UP | IFF_RUNNING ) )
			   != ( IFF_UP | IFF_RUNNING ) )
	    continue;
#endif

	// skip an interface if we can't get its address
	if( INET_ioctl( sock,
			SIOCGIFADDR, //lint !e569 !e912 !e917
			(int *)&ifr ) != 0 ) //lint !e740
	    continue;

	// we are only interested in internet interfaces
	if( ifr.ifr_addr.sa_family != AF_INET )
	    continue;

	// this interface passes

	sockaddr_in * ip = (sockaddr_in *)&ifr.ifr_addr; //lint !e740

	RTMemoryUtil::memcpy( address, &ip->sin_addr, (int)sizeof( *address ) );

	return 1;
    }

    return 0;
}
