/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#ifdef PRAGMA
#pragma implementation "RTThreadInfo.h"
#endif

#include <RTDiag.h>
#include <RTJob.h>
#include <RTThread.h>
#include <RTThreadInfo.h>

static void * thread_run( void * job )
{
    ((RTJob *)job)->mainLoop();

    return (void *)0;
}

RTThread::RTThread( RTJob * job, int stack_size, int prio )
    : _job ( job )
    , _info( new RTThreadInfo )
{
    pthread_attr_t attr;

    if( pthread_attr_init( &attr ) == -1 )
	RTDiag::panic( "pthread_attr_create" );

    pthread_attr_setstacksize( &attr, stack_size );
    pthread_attr_setdetachstate( &attr, PTHREAD_CREATE_JOINABLE );

    if( pthread_create( &_info->tid, &attr, thread_run, job ) != 0 )
	RTDiag::panic( "pthread_create" );
    else
	_info->valid = 1;

    (void)prio; // no pthread_attr_setprio() or pthread_setprio()
}

RTThreadInfo::RTThreadInfo( void )
    : valid( 0 )
{
}

RTThreadInfo::~RTThreadInfo( void )
{
    if( valid == 0 )
	return; // never successfully created

    void * status = (void *)0;

    if( pthread_join( tid, &status ) != 0 )
	RTDiag::panic( "pthread_join" );
}
