/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTDiag.h>
#include <RTMain.h>
#include <RTWinCE.h>

#include <commctrl.h>

static char * toAscii( const TCHAR * unicode )
{
    const TCHAR * reader;
    char        * writer;

    for( reader = unicode; *reader != TEXT( '\0' ); ++reader )
	if( (unsigned)*reader >= 0x80U )
	    return (char *)0; // a non-ASCII character

    int    length = reader - unicode;
    char * ascii  = new char [ (unsigned)length + 1U ];

    if( ascii != (char *)0 )
    {
	for( reader = unicode, writer = ascii; *reader != TEXT( '\0' ); )
	    *writer++ = (char)*reader++;
	*writer = '\0';
    }

    return ascii;
}

static char * parseArgs( const TCHAR  *   buffer,
			 int            & argc,
			 const char  ** & argv )
{
    int    escaped;
    int    quoted;
    char * reader;
    char * writer;
    char * ascii = toAscii( buffer );

    if( ascii == (char *)0 )
	return (char *)0;

    for( argc = 0, reader = ascii; ; )
    {
	while( *reader == ' ' )
	    ++reader;
	if( *reader == '\0' )
	    break;
	++argc;

	escaped = 0;
	quoted  = 0;
	for(;;)
	{
	    if( *reader == '\0' )
		break;

	    if( *reader == '"' )
		quoted = ! quoted;
	    else if( *reader == ' ' && ! quoted )
		break;
	    ++reader;
	}
    }

    if( (argv = new const char * [ (unsigned)argc + 2U ]) == (const char **)0 )
    {
	// Not enough memory: use default arguments.
	delete [] ascii;
	return (char *)0;
    }

    // Insert the application name.
    argv[ 0 ] = "TargetRTS";

    argc   = 0;
    reader = ascii;
    writer = ascii;
    quoted = 0;
    for(;;)
    {
	while( *reader == ' ' )
	    ++reader, *writer++ = '\0';
	if( *reader == '\0' )
	{
	    *writer = '\0';
	    break;
	}
	argv[ ++argc ] = reader;
	writer = reader;
	for(;;)
	{
	    if( *reader == '\0' )
		break;

	    if( *reader == '"' )
	    {
		quoted = ! quoted;
		++reader;
	    }
	    else if( *reader == ' ' && ! quoted )
		break;
	    else
		*writer++ = *reader++;
	}
    }
    argv[ ++argc ] = (char *)0;

    return ascii;
}

#if USE_WINCE_RTCONSOLE

static LRESULT CALLBACK wndProc( HWND   window,
				 UINT   message,
				 WPARAM wParam,
				 LPARAM lParam )
{
    switch( message )
    {
    case WM_SETFOCUS:
	// Give focus to the terminal window
	SetFocus( RTWinCE::soleInstance->hTermWnd );
	break;

    case WM_SIZE:
    {
	RTWinCE * ce = RTWinCE::soleInstance;
	RECT      rc;

	GetClientRect( window, &rc );
#ifdef UNDER_CE
	rc.top += CommandBar_Height( ce->hCmdBar );
#endif
	SetWindowPos( ce->hTermWnd, HWND_TOP,
		      rc.left,                rc.top,
		      rc.right - rc.left + 1, rc.bottom - rc.top + 1,
		      0 );
	ShowWindow( ce->hTermWnd, SW_SHOWNORMAL );
	break;
    }

    case WM_CLOSE:
	SendMessage( RTWinCE::soleInstance->hTermWnd,
		     WM_CLOSE, (WPARAM)0, (LPARAM)0 );
	DestroyWindow( window );
	break;

    case WM_DESTROY:
	PostQuitMessage( 0 );
	break;

    default:
	return DefWindowProc( window, message, wParam, lParam );
    }

    return 0;
}

static const TCHAR MainClassName[]    = TEXT( "TargetRTS" );
static const TCHAR ConsoleClassName[] = TEXT( "RTConsole" );

static BOOL initApplication( HINSTANCE hInstance )
{
    WNDCLASS wc;

    wc.style         = 0;
    wc.lpfnWndProc   = wndProc;
    wc.cbClsExtra    = 0;
    wc.cbWndExtra    = 0;
    wc.hInstance     = hInstance;
    wc.hIcon         = (HICON)0;
#ifdef UNDER_CE
    wc.hCursor       = (HCURSOR)0;
#else
    wc.hCursor       = LoadCursor( (HINSTANCE)0, IDC_ARROW );
#endif
    wc.hbrBackground = (HBRUSH)GetStockObject( WHITE_BRUSH );
    wc.lpszMenuName  = (const TCHAR *)0;
    wc.lpszClassName = MainClassName;

    if( ! RegisterClass( &wc ) )
	return FALSE;

    wc.lpfnWndProc   = &RTConsole::WndProc;
    wc.lpszClassName = ConsoleClassName;
    wc.cbWndExtra    = sizeof( RTConsole * );

    if( ! RegisterClass( &wc ) )
	return FALSE;

    return TRUE;
}

static BOOL initInstance( HINSTANCE hInstance, int nShowCmd )
{
    HWND hMainWnd;
    HWND hCmdBar;
    HWND hTermWnd;
    RECT rc;

    // Create the main window.

    hMainWnd = CreateWindowEx( 0,
			       MainClassName,
			       TEXT( "Log" ),
#ifdef WS_OVERLAPPEDWINDOW
			       WS_OVERLAPPEDWINDOW |
#endif
			       WS_CLIPCHILDREN,
			       CW_USEDEFAULT, 0,
			       CW_USEDEFAULT, 0,
			       (HWND)0, (HMENU)0,
			       hInstance, (void *)0 );

    // If the main window cannot be created, terminate the application.

    if( hMainWnd == (HWND)0 )
	return FALSE;

    GetClientRect( hMainWnd, &rc );

#ifdef UNDER_CE
    hCmdBar = CommandBar_Create( hInstance, hMainWnd, 1 );
    CommandBar_InsertMenubar( hCmdBar, hInstance, IDR_MAIN_MENU, 0 );
    CommandBar_AddAdornments( hCmdBar, 0, 0 );
    rc.top += CommandBar_Height( hCmdBar );
#else
    hCmdBar = (HWND)0;
    // NT?
#endif

    // Create the terminal window.
    hTermWnd = CreateWindowEx( 0,
			       ConsoleClassName,
			       TEXT( "IO" ),
			       WS_CHILD | WS_VSCROLL | WS_HSCROLL
					| WS_VISIBLE | WS_BORDER,
			       rc.left,            rc.top,
			       rc.right - rc.left, rc.bottom - rc.top,
			       hMainWnd, (HMENU)0,
			       hInstance, (void *)0 );

    if( hTermWnd == (HWND)0 )
    {
	DestroyWindow( hMainWnd );
	return FALSE;
    }

    SetWindowLong( hTermWnd, 0, (LONG)new RTConsole( hTermWnd, 50, 80 ) );

//  RTWinCE::soleInstance->hMainWnd   = hMainWnd;
    RTWinCE::soleInstance->hCmdBar    = hCmdBar;
    RTWinCE::soleInstance->hTermWnd   = hTermWnd;
    RTWinCE::soleInstance->mainThread = (HANDLE)0;

    // Show the window and paint its contents.

    ShowWindow( hMainWnd, nShowCmd );
    UpdateWindow( hMainWnd );

    SetFocus( hTermWnd );

    return TRUE;
}

extern "C" int WINAPI wWinMain( HINSTANCE hInstance,
				HINSTANCE hPrevInstance,
				LPTSTR    lpCmdLine,
				int       nShowCmd )
{
    RTWinCE ce;

    RTWinCE::soleInstance = &ce;

    // Register our window classes.
    if( hPrevInstance == (HINSTANCE)0 && ! initApplication( hInstance ) )
	return 0;

    // Create our windows.
    if( ! initInstance( hInstance, nShowCmd ) )
	return 0;

    // Parse command-line.
    int           argc      = 0;
    const char ** argv      = (const char **)0;
    char        * argBuffer = parseArgs( lpCmdLine, argc, argv );

    // If there are no arguments pretend that they cannot be passed and
    // try to read the default arguments specified in the component.
    if( argc <= 1 )
	argc = 0;

    // Start the application threads.
    (void)RTMain::entryPoint( argc, argv );

    // Start the message loop.
    MSG msg;

    while( GetMessage( &msg, (HWND)0, 0, 0 ) )
    {
	TranslateMessage( &msg );
	DispatchMessage( &msg );
    }

    delete [] argBuffer;

    RTWinCE::soleInstance = (RTWinCE *)0;

    // Return the exit code to the system.
    return msg.wParam;
}

#else

extern "C" int WINAPI wWinMain( HINSTANCE hInstance,
				HINSTANCE hPrevInstance,
				LPTSTR    lpCmdLine,
				int       nShowCmd )
{
    // Parse command-line.
    int           argc      = 0;
    const char ** argv      = (const char **)0;
    char        * argBuffer = parseArgs( lpCmdLine, argc, argv );

    // If there are no arguments pretend that they cannot be passed and
    // try to read the default arguments specified in the component.
    if( argc <= 1 )
	argc = 0;

    // Start the application threads.
    int ret_code = RTMain::entryPoint( argc, argv );

    delete [] argBuffer;

    return ret_code;
}

#endif // USE_WINCE_RTCONSOLE
