/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTWinCE.h>

void RTConsole::paint( void )
{
    HDC           hDC;
    PAINTSTRUCT   ps;
    RECT        & rc = ps.rcPaint;
    HGDIOBJ       oldFont;

    makeCaret( CaretHidden );
    hDC = BeginPaint( window, &ps );

  {
    oldFont = (HFONT)SelectObject( hDC, font );
    (void)SetTextColor( hDC, RGB( 0, 0, 0 ) );
    (void)SetBkColor( hDC, RGB( 0xFF, 0xFF, 0xFF ) );

    // Translate region to rows and columns.

    int first_row = screen.top + rc.top    / character.height;
    int last_row  = screen.top + rc.bottom / character.height + 1;

    int first_col = screen.left + rc.left  / character.width;
    int last_col  = screen.left + rc.right / character.width + 1;

    int text_cols = buffer.cols;

    if( text_cols >= last_col )
	text_cols  = last_col;
    else
    {
	HBRUSH hBrush = (HBRUSH)GetStockObject( WHITE_BRUSH );
	RECT   margin;

	// clear the area to the right of the last column in the buffer
	margin.top    = rc.top;
	margin.left   = ( buffer.cols - screen.left ) * character.width;
	margin.bottom = ( buffer.rows - screen.top  ) * character.height;
	margin.right  = rc.right;

	FillRect( hDC, &margin, hBrush );
	DeleteObject( hBrush );
    }

    text_cols -= first_col;

    int x = ( first_col - screen.left ) * character.width;
    int y = ( first_row - screen.top  ) * character.height;

    for( int row  = first_row;
	     row  < last_row;
	     row += 1,         y += character.height )
    {
	if( row < buffer.rows && text_cols >= 0 )
	{
	    ExtTextOut( hDC, x, y,
			0, (const RECT *)0,
			&buffer.data[ row * buffer.cols + first_col ],
			text_cols,
			(const int *)0 );
	}
	else
	{
	    HBRUSH hBrush = (HBRUSH)GetStockObject( WHITE_BRUSH );
	    RECT   margin;

	    // clear the area below last the last row in the buffer
	    margin.top    = y;
	    margin.left   = rc.left;
	    margin.bottom = rc.bottom;
	    margin.right  = rc.right;

	    FillRect( hDC, &margin, hBrush );
	    DeleteObject( hBrush );
	    break;
	}
    }

    if( oldFont != NULL )
	SelectObject( hDC, oldFont );
  }

    EndPaint( window, &ps );
    makeCaret( CaretVisible );
}
