#! perl -w

# Licensed Materials - Property of IBM                               
# Component IBM Rational ROSE_RT BUILD : TARGET_RTS                 
# Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
# U.S. Government Users Restricted Rights -  Use, duplication or     
# disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.

# This script performs two functions: It translates object file lists
# (identified by %file on the command line) to linker command files,
# and adds the constructor/destructor tables to the executable.
#
# Example usage:
# ===============
# perl link.pl ARCH=386 -o TopCapsule \
#   main.o %OLIST user_obj.o -LX -lx ...
#
# The general algorithm:
#   Translate each OLIST to OLIST.cmd
#   ld386 -r -o RTmunch.o main.o -T OLIST.cmd user_obj.o -LX -lx
#   nm386 RTmunch.o | munch > RTctor.c
#   cc386 -o RTctor.o -c RTctor.c
#   ld386 -r -o TopCapsule RTctor.o RTmunch.o

# Win9x command.com does not interact with perl according to win32 standard.
# We therefore need to call some functions in a non-standard way.
#
# This detects if the system is Win95 for that purpose.

sub mySystem
{
    my( $OS, $ROSERT_HOST );

    $OS          = $ENV{'OS'}          || '?';
    $ROSERT_HOST = $ENV{'ROSERT_HOST'} || '?';

    if( $ROSERT_HOST eq 'win32' && $OS !~ m{^Windows_NT$}i )
    {
        return system( join( ' ', @_ ) );
    }
    else
    {
        return system( @_ );
    }
}

sub report_ges_error
{
    my( $ERROR ) = @_;

    print "description '$ERROR' severity 'error'\n";
    print "result 'failure'\n";

    exit 1;
}

sub show_usage
{
    &report_ges_error( "Usage: $0 ARCH=<arch> options ... " );
}

&show_usage() unless $#ARGV >= 1;

if( $ARGV[0] =~ m{^ARCH=(\w+)$} )
{
    
    $arch = $1;
    shift @ARGV;
}
else
{
    &show_usage();
}

&report_ges_error( "WIND_USR is not set" )
	unless( $WIND_USR = $ENV{'WIND_USR'} );

# commands
$cc_cmd  = "c++$arch";
$output  = 'a.out';

$status   = 0;                              # assume all is well
$cmdfile  = 'ld000';                        # command filename pattern
@tmpfiles = (); # temporary files

# Translate all indirect object file lists (identified by %) into linker
# command files for use in the prelink step.

@command  = ();

{
	@script_intro = ();
}



foreach $arg ( @ARGV )
{

    if( $arg eq '-o' )
    {
	$lasto = 1;
    }
    elsif( $lasto )
    {
	$output = $arg;
	$lasto  = 0;
    }
    elsif( $arg =~ m{^%(.*)$} )
    {
	$in = $1;
	if( ! open( IN, "<$in" ) )
	{
	    print STDERR "link.pl: Can't read '$in': $!\n";
	    $status = 1;
	}
	else
	{
	    $out = "$cmdfile.cmd";
	    ++$cmdfile;
	    if( ! open( OUT, ">$out" ) )
	    {
		print STDERR "link.pl: Can't create '$out': $!\n";
		$status = 1;
	    }
	    else
	    {
		while( <IN> )
		{
		    chomp;
		    print OUT "INPUT(", $_, ".o)\n";
		}
		close( OUT );

		push @command, @script_intro, $out;
		@script_intro = ();
		push @tmpfiles, $out;
	    }
	    close( IN );
	}
    }
    else
    {
		push @command, $arg;
    }
}


# compile the constructor/destructor tables
if ( $arch eq 'pentium' )
{
  @mycommand = ( $cc_cmd, '-mcpu=i486', '-march=i486', '-ansi', '-mrtp', '-O2', '-fno-builtin', 
               '-fno-defer-pop', '-Wall', '-DCPU=SIMPENTIUM', '-DTOOL_FAMILY=gnu', '-DTOOL=gnu');
  @LinkDirs	=	('-L'.$WIND_USR.'\\lib\\simpentium\\SIMPENTIUM\\common/','-o', $output);
  push @mycommand, @command;
  push @mycommand, @LinkDirs;
  $status = &mySystem( @mycommand ) if $status == 0;					
}
else
{
  @mycommand = ( $cc_cmd, '-mcpu=603', '-ansi', '-mrtp', '-O2', '-fno-builtin', 
               '-fno-defer-pop', '-Wall', '-DCPU=PPC603', '-DTOOL_FAMILY=gnu', '-DTOOL=gnu');
  @LinkDirs	=	('-L'.$WIND_USR.'\\lib\\ppc\\PPC32\\common/','-o', $output);
  push @mycommand, @command;
  push @mycommand, @LinkDirs;
  $status = &mySystem( @mycommand ) if $status == 0;	
}

exit $status if $status < 256;
exit $status / 256;
